@@ -4,7 +4,7 @@ use warnings;
 use strict;
 use 5.10.1;
 
-our $VERSION = '0.001_02';
+our $VERSION = '0.002_01';
 use Exporter 'import';
 our @EXPORT_OK = qw( print_table );
 
@@ -22,58 +22,102 @@ no warnings 'utf8';
 
 use constant CLEAR_SCREEN   => "\e[1;1H\e[0J";
 
+sub new {
+    return bless {}, shift if @_ == 1;
+    return bless $_[1], $_[0] if eval { $_[1]->{db_browser_mode} };
+    my ( $class, $opt ) = @_;
+    my $arg = scalar( @_ ) - 1;
+    croak "new: called with $arg arguments - 0 or 1 arguments expected." if $arg > 1;
+    croak "new: The (optional) argument is not a HASH reference." if ref $opt ne 'HASH';
+    my $self = bless {}, $class;
+    $self->__validate_options( $opt );
+    return $self;
+}
 
-sub print_table {
-    my ( $a_ref, $opt, $private ) = @_;
-    if ( ! $private ) {
-        croak "print_table: called with " . scalar @_ . " arguments - 1 or 2 arguments expected." if @_ < 1 || @_ > 2;
-        croak "print_table: Required an ARRAY reference as the first argument."      if ref $a_ref  ne 'ARRAY';
-        croak "print_table: The (optional) second argument is not a HASH reference." if defined $opt && ref $opt ne 'HASH';
-        $opt->{progress_bar} = 20000 if ! exists $opt->{progress_bar};
-        $opt->{max_rows}     = 50000 if ! exists $opt->{max_rows};
-        $opt->{tab_width}     //= 2;
-        $opt->{min_col_width} //= 30;
-        $opt->{table_expand}  //= 1;
-        $opt->{binary_filter} //= 0;
-        $opt->{undef}         //= '';
-        $opt->{mouse}         //= 0;
-        $opt->{binary_string} //= 'BNRY';
-        #$opt->{header_row}    //= 1;
-        #$opt->{choose_colums} //= 0;
-        $opt->{thsd_sep} = ',';
-        for my $int ( qw( progress max_rows min_col_width tab_width ) ) { # choose_colums
-            next if ! defined $opt->{$int};
-            croak "print_table: '$opt->{$int}' is not a valid value for option '$int'." if $opt->{$int} !~ /^\d+\z/;
-        }
-        for my $bol ( qw( table_expand binary_filter ) ) { #  header_row
-            croak "print_table: '$opt->{$bol}' is not a valid value for option '$bol'." if $opt->{$bol} !~ /^[01]\z/;
-        }
-        croak "print_table: '$opt->{mouse}' is not a valid value for option 'mouse'." if $opt->{mouse} !~ /^[01234]\z/;
-        # offer to choose columns if $opt->{choose_colums};
+sub __validate_options {
+    my ( $self, $opt ) = @_;
+    return if ! defined $opt;
+    for my $key ( qw( progress max_rows min_col_width tab_width ) ) { # choose_columns
+        next if ! defined $opt->{$key};
+        croak "print_table: '$opt->{$key}' is not a valid value for option '$key'." if $opt->{$key} !~ /^\d+\z/;
+        $self->{$key} = $opt->{$key};
     }
+    for my $key ( qw( table_expand binary_filter ) ) { #  header_row
+        next if ! defined $opt->{$key};
+        croak "print_table: '$opt->{$key}' is not a valid value for option '$key'." if $opt->{$key} !~ /^[01]\z/;
+        $self->{$key} = $opt->{$key};
+    }
+    for my $key ( qw( mouse ) ) {
+        next if ! defined $opt->{$key};
+        croak "print_table: '$opt->{$key}' is not a valid value for option '$key'." if $opt->{$key} !~ /^[01234]\z/;
+        $self->{$key} = $opt->{$key};
+    }
+}
 
-    my $info = {};
-    my $gcs_bnry = Unicode::GCString->new( $opt->{binary_string} );
-    $info->{binary_length} = $gcs_bnry->columns;
-    if ( defined $opt->{progress_bar} ) {
-        say 'Computing: ...';
-        if ( @$a_ref * @{$a_ref->[0]} > $opt->{progress_bar} ) {
-            $info->{show_progress} = 1;
-        }
-    }
-    $info = _calc_col_width( $info, $opt, $a_ref );
-    _inner_print_tbl( $info, $opt, $a_ref );
+sub __set_defaults {
+    my ( $self ) = @_;
+    $self->{progress_bar} = 20000 if ! exists $self->{progress_bar};
+    $self->{max_rows}     = 50000 if ! exists $self->{max_rows};
+    $self->{tab_width}      //= 2;
+    $self->{min_col_width}  //= 30;
+    $self->{table_expand}   //= 1;
+    $self->{binary_filter}  //= 0;
+    $self->{undef}          //= '';
+    $self->{mouse}          //= 0;
+    $self->{binary_string}  //= 'BNRY';
+    #$self->{header_row}     //= 1;
+    #$self->{choose_columns} //= 0;
+    $self->{thsd_sep} = ',';
 }
 
 
-sub _inner_print_tbl {
-    my ( $info, $opt, $a_ref ) = @_;
+sub print_table {
+    if ( ref $_[0] ne 'Term::TablePrint' ) {
+        return Term::TablePrint->new( $_[1] )->print_table( $_[0] );
+    }
+    my ( $self, $a_ref, $opt ) = @_;
+    if ( ! $self->{db_browser_mode} ) {
+        my $arg = scalar( @_ ) - 1;
+        croak "print_table: called with $arg arguments - 1 or 2 arguments expected." if $arg < 1 || $arg > 2;
+        croak "print_table: Required an ARRAY reference as the first argument."      if ref $a_ref  ne 'ARRAY';
+        croak "print_table: The (optional) second argument is not a HASH reference." if defined $opt && ref $opt ne 'HASH';
+        $self->{backup_opt} = { map{ $_ => $self->{$_} } keys %$opt } if defined $opt;
+        $self->__validate_options( $opt );
+        $self->__set_defaults();
+        #if ( $self->{choose_columns} ) {
+        #    my @i = choose( $a_ref->[0], { prompt => 'Choose: ', index => 1 } );
+        #    for my $i ( 0 .. $#$a_ref ) {
+        #        $a_ref->[$i] = [ @{$a_ref->[$i]}[sort @i] ];
+        #    }
+        #}
+    }
+    my $gcs_bnry = Unicode::GCString->new( $self->{binary_string} );
+    $self->{binary_length} = $gcs_bnry->columns;
+    if ( defined $self->{progress_bar} ) {
+        say 'Computing: ...';
+        if ( @$a_ref * @{$a_ref->[0]} > $self->{progress_bar} ) {
+            $self->{show_progress} = 1;
+        }
+    }
+    $self->__calc_col_width( $a_ref );
+    $self->__inner_print_tbl( $a_ref );
+    if ( $self->{backup_opt} ) {
+        my $backup_opt = delete $self->{backup_opt};
+        for my $key ( keys %$backup_opt ) {
+            $self->{$key} = $backup_opt->{$key};
+        }
+    }
+}
+
+
+sub __inner_print_tbl {
+    my ( $self, $a_ref ) = @_;
     my ( $term_width ) = _term_size();
-    my $width_cols = _calc_avail_width( $info, $opt, $a_ref, $term_width );
+    my $width_cols = $self->__calc_avail_width( $a_ref, $term_width );
     return if ! $width_cols;
-    my ( $list, $len ) = _trunk_col_to_avail_width( $info, $opt, $a_ref, $width_cols );
-    if ( $opt->{max_rows} && @$list > $opt->{max_rows} ) {
-        my $reached_limit = 'REACHED LIMIT "MAX_ROWS": ' . _insert_sep( $opt->{max_rows}, $opt->{thsd_sep} );
+    my ( $list, $len ) = $self->__trunk_col_to_avail_width( $a_ref, $width_cols );
+    if ( $self->{max_rows} && @$list > $self->{max_rows} ) {
+        my $reached_limit = 'REACHED LIMIT "MAX_ROWS": ' . _insert_sep( $self->{max_rows}, $self->{thsd_sep} );
         my $gcs = Unicode::GCString->new( $reached_limit );
         if ( $gcs->columns > $term_width ) {
             $reached_limit = _unicode_sprintf( $term_width, 'REACHED LIMIT', 0 );
@@ -85,16 +129,16 @@ sub _inner_print_tbl {
     while ( 1 ) {
         if ( ( _term_size() )[0] != $width ) {
             ( $width ) = _term_size();
-            _inner_print_tbl( $info, $opt, $a_ref );
+            $self->__inner_print_tbl( $a_ref );
             return;
         }
         my $row = choose(
             $list,
-            { prompt => '', index => 1, default => $old_row, ll => $len, layout => 3, clear_screen => 1, mouse => $opt->{mouse} }
+            { prompt => '', index => 1, default => $old_row, ll => $len, layout => 3, clear_screen => 1, mouse => $self->{mouse} }
         );
         return if ! defined $row;
         return if $row == 0;
-        if ( ! $opt->{table_expand} ) {
+        if ( ! $self->{table_expand} ) {
             $old_row = 0;
             next;
         }
@@ -103,25 +147,23 @@ sub _inner_print_tbl {
             next;
         }
         $old_row = $row;
-        my $row_data = _single_row( $a_ref, $row, $info->{longest_col_name} + 1 );
+        my $row_data = $self->__single_row( $a_ref, $row, $self->{longest_col_name} + 1 );
         choose(
             $row_data,
-            { prompt => '', layout => 3, clear_screen => 1, mouse => $opt->{mouse} }
+            { prompt => '', layout => 3, clear_screen => 1, mouse => $self->{mouse} }
         );
     }
 }
 
 
-sub _single_row {
-    my ( $a_ref, $row, $length_key ) = @_;
+sub __single_row {
+    my ( $self, $a_ref, $row, $length_key ) = @_;
     my ( $term_width ) = _term_size();
     $length_key = int( $term_width / 100 * 33 ) if $length_key > int( $term_width / 100 * 33 );
     my $separator = ' : ';
     my $gcs = Unicode::GCString->new( $separator );
     my $length_sep = $gcs->columns;
-    #my $header_row = $length_key > 1 ? 1 : 0;
-    #$length_key = length( scalar @{$a_ref->[0]} ) if ! $header_row;
-    ##$separator = '' if ! $header_row;
+    #$length_key = length( scalar @{$a_ref->[0]} ) if ! $self->{header_row};
     my $col_max = $term_width - ( $length_key + $length_sep + 1 );
     my $line_fold = Text::LineFold->new(
         Charset=> 'utf8',
@@ -133,8 +175,7 @@ sub _single_row {
     for my $col ( 0 .. $#{$a_ref->[0]} ) {
         push @{$row_data}, ' ';
         my $key = $a_ref->[0][$col];
-        ##$key = '' if ! $header_row;
-        #$key = $col + 1 if ! $header_row;
+        #$key = $col + 1 if ! $self->{header_row};
         my $sep = $separator;
         if ( ! defined $a_ref->[$row][$col] || $a_ref->[$row][$col] eq '' ) {
             push @{$row_data}, sprintf "%*.*s%*s%s", $length_key, $length_key, $key, $length_sep, $sep, '';
@@ -152,24 +193,25 @@ sub _single_row {
 }
 
 
-sub _calc_col_width {
-    my ( $info, $opt, $a_ref ) = @_;
+sub __calc_col_width {
+    my ( $self, $a_ref ) = @_;
     my $binray_regexp = qr/[\x00-\x08\x0B-\x0C\x0E-\x1F]/;
-    $info->{longest_col_name} = 0;
+    $self->{longest_col_name} = 0;
     my $normal_row = 0;
-    #$normal_row = 1 if ! $opt->{header_row};
-    $info->{width_cols} = [ ( 1 ) x @{$a_ref->[0]} ];
-    #$info->{width_head} = [ ( 1 ) x @{$a_ref->[0]} ] if ! $opt->{header_row};
+    #$normal_row = 1 if ! $self->{header_row};
+    $self->{width_cols} = [ ( 1 ) x @{$a_ref->[0]} ];
+    #$self->{width_head} = [ ( 1 ) x @{$a_ref->[0]} ] if ! $self->{header_row};
+    my @col_idx = ( 0 .. $#{$a_ref->[0]} );
     for my $row ( @$a_ref ) {
-        for my $i ( 0 .. $#$row ) {
-            $row->[$i] //= $opt->{undef};
+        for my $i ( @col_idx ) {
+            $row->[$i] //= $self->{undef};
             if ( ref $row->[$i] ) {
-                $row->[$i] = _handle_reference( $row->[$i] );
+                $row->[$i] = $self->__handle_reference( $row->[$i] );
             }
             my $width;
-            if ( $opt->{binary_filter} && substr( $row->[$i], 0, 100 ) =~ $binray_regexp ) {
-                $row->[$i] = $opt->{binary_string};
-                $width     = $info->{binary_length};
+            if ( $self->{binary_filter} && substr( $row->[$i], 0, 100 ) =~ $binray_regexp ) {
+                $row->[$i] = $self->{binary_string};
+                $width     = $self->{binary_length};
             }
             else {
                 $row->[$i] =~ s/^\p{Space}+//;
@@ -181,26 +223,25 @@ sub _calc_col_width {
                 $width = $gcs->columns;
             }
             if ( $normal_row ) {
-                $info->{width_cols}[$i] = $width if $width > $info->{width_cols}[$i];
-                ++$info->{not_a_number}[$i] if $row->[$i] && ! looks_like_number $row->[$i];
+                $self->{width_cols}[$i] = $width if $width > $self->{width_cols}[$i];
+                ++$self->{not_a_number}[$i] if $row->[$i] && ! looks_like_number $row->[$i];
             }
             else {
                 # column name
-                $info->{width_head}[$i] = $width;
-                $info->{longest_col_name} = $width if $width > $info->{longest_col_name};
+                $self->{width_head}[$i] = $width;
+                $self->{longest_col_name} = $width if $width > $self->{longest_col_name};
                 $normal_row = 1 if $i == $#$row;
             }
         }
     }
-    return $info;
 }
 
 
-sub _calc_avail_width {
-    my ( $info, $opt, $a_ref, $term_width ) = @_;
-    my $width_head = [ @{$info->{width_head}} ];
-    my $width_cols = [ @{$info->{width_cols}} ];
-    my $avail_width = $term_width - $opt->{tab_width} * $#$width_cols;
+sub __calc_avail_width {
+    my ( $self, $a_ref, $term_width ) = @_;
+    my $width_head = [ @{$self->{width_head}} ];
+    my $width_cols = [ @{$self->{width_cols}} ];
+    my $avail_width = $term_width - $self->{tab_width} * $#$width_cols;
     my $sum = sum( @$width_cols );
     if ( $sum < $avail_width ) {
         # auto cut
@@ -219,18 +260,18 @@ sub _calc_avail_width {
         return $width_head, $width_cols;
     }
     elsif ( $sum > $avail_width ) {
-        my $minimum_with = $opt->{min_col_width} || 1;
+        my $minimum_with = $self->{min_col_width} || 1;
         if ( @$width_head > $avail_width ) {
             say 'Terminal window is not wide enough to print this table.';
             choose(
                 [ 'Press ENTER to show the column names.' ],
-                { prompt => '', clear_screen => 0, mouse => $opt->{mouse} }
+                { prompt => '', clear_screen => 0, mouse => $self->{mouse} }
             );
             my $prompt = 'Reduce the number of columns".' . "\n";
             $prompt .= 'Close with ENTER.';
             choose(
                 $a_ref->[0],
-                { prompt => $prompt, clear_screen => 0, mouse => $opt->{mouse} }
+                { prompt => $prompt, clear_screen => 0, mouse => $self->{mouse} }
             );
             return;
         }
@@ -276,13 +317,13 @@ sub _calc_avail_width {
 }
 
 
-sub _trunk_col_to_avail_width {
-    my ( $info, $opt, $a_ref, $width_cols ) = @_;
+sub __trunk_col_to_avail_width {
+    my ( $self, $a_ref, $width_cols ) = @_;
     my $total = $#{$a_ref};                   #
     my $next_update = 0;                      #
     my $c = 0;                                #
     my $progress;                             #
-    if ( $info->{show_progress} ) {           #
+    if ( $self->{show_progress} ) {           #
         local $| = 1;                         #
         print CLEAR_SCREEN;                   #
         $progress = Term::ProgressBar->new( { #
@@ -290,17 +331,18 @@ sub _trunk_col_to_avail_width {
             count => $total,                  #
             remove => 1 } );                  #
         $progress->minor( 0 );                #
-    }                                         #
+    }
+    #unshift @$a_ref, [ map { '=' x $width_cols->[$_] } 0 .. $#$width_cols ] if ! $self->{header_row};
     my $list;
-    my $tab = ' ' x $opt->{tab_width};
+    my $tab = ' ' x $self->{tab_width};
     for my $row ( @$a_ref ) {
         my $str = '';
         for my $i ( 0 .. $#$width_cols ) {
-            $str .= _unicode_sprintf( $width_cols->[$i], $row->[$i], $info->{not_a_number}[$i] ? 0 : 1 );
+            $str .= _unicode_sprintf( $width_cols->[$i], $row->[$i], $self->{not_a_number}[$i] ? 0 : 1 );
             $str .= $tab if $i != $#$width_cols;
         }
         push @$list, $str;
-        if ( $info->{show_progress} ) {                                      #
+        if ( $self->{show_progress} ) {                                      #
             my $is_power = 0;                                                #
             for ( my $i = 0; 2 ** $i <= $c; ++$i ) {                         #
                 $is_power = 1 if 2 ** $i == $c;                              #
@@ -309,28 +351,14 @@ sub _trunk_col_to_avail_width {
             ++$c;                                                            #
         }                                                                    #
     }
-    $progress->update( $total ) if $info->{show_progress} && $total >= $next_update; #
-    my $len = sum( @$width_cols, $opt->{tab_width} * $#{$width_cols} );
+    $progress->update( $total ) if $self->{show_progress} && $total >= $next_update; #
+    my $len = sum( @$width_cols, $self->{tab_width} * $#{$width_cols} );
     return $list, $len;
 }
 
 
-sub _minus_x_percent {
-    my ( $value, $percent ) = @_;
-    my $new = int( $value - ( $value / 100 * $percent ) );
-    return $new > 0 ? $new : 1;
-}
-
-
-sub _term_size {
-    my ( $width, $heigth ) = chars();
-    return $width - 1, $heigth if $^O eq 'MSWin32';
-    return $width, $heigth;
-}
-
-
-sub _handle_reference {
-    my ( $ref ) = @_;
+sub __handle_reference {
+    my ( $self, $ref ) = @_;
     if ( ref $ref eq 'ARRAY' ) {
         return 'ref: [' . join( ',', map { '"' . $_ . '"' } @$ref ) . ']';
     }
@@ -355,6 +383,21 @@ sub _handle_reference {
 }
 
 
+
+sub _minus_x_percent {
+    my ( $value, $percent ) = @_;
+    my $new = int( $value - ( $value / 100 * $percent ) );
+    return $new > 0 ? $new : 1;
+}
+
+
+sub _term_size {
+    my ( $width, $heigth ) = chars();
+    return $width - 1, $heigth if $^O eq 'MSWin32';
+    return $width, $heigth;
+}
+
+
 sub _insert_sep {
     my ( $number, $separator ) = @_;
     return if ! defined $number;
@@ -407,33 +450,28 @@ Term::TablePrint - Print a table on the terminal.
 
 =head1 VERSION
 
-Version 0.001_02
+Version 0.002_01
 
 =cut
 
 =head1 SYNOPSIS
 
-    use Term::TablePrint qw( print_table );
-
     my $table = [ [ 'id', 'name' ],
                   [    1, 'Ruth' ],
                   [    2, 'John' ],
                   [    3, 'Mark' ],
                   [    4, 'Nena' ], ];
 
+    use Term::TablePrint qw( print_table );
+
     print_table( $table );
 
+    # or OO style:
 
-    use DBI;
+    use Term::TablePrint;
 
-    my $dbh = DBI->connect( ... );
-    my $sth = $dbh->prepare( "SELECT * FROM table" );
-    $sth->execute();
-    my $a_ref = $sth->fetchall_arrayref();
-    unshift @$a_ref, $sth->{NAME};
-
-    my $opt = {};
-    print_table( $a_ref, $opt );
+    my $pt = Term::TablePrint->new();
+    $pt->print_table( $table );
 
 =head1 DESCRIPTION
 
@@ -458,16 +496,39 @@ one line preceded by the column name. This might be useful if the columns were c
 To get a proper output, C<print_table> uses the C<columns> method from L<Unicode::GCString> to calculate the string
 length.
 
+=head1 METHODES
+
+=head2 new
+
+Returns an object. As an argument it can be passed  a reference to a hash which holds the options as pairs of
+"option-name" and "option-value".
+
+    my $tp = Term::TablePrint->new( [ \%options ] );
+
+=head2 print_table
+
+Prints the table passed with the first argument.
+
+    $tp->print_table( $array_ref, [ \%options ] );
+
+The first argument must be a reference to an array of arrays. The first array of the arrays holds the column names. The
+following arrays are the table rows where the elements of these arrays are the field values.
+
+As a second and optional argument can be passed a reference to a hash which holds the options as pairs of
+"option-name" and "option-value".
+
 =head1 SUBROUTINES
 
 =head2 print_table
 
-    print_table( $array_ref, [ \%options ] )
+Prints the table passed with the first argument.
 
-The first argument must be a reference to an array of arrays. The first array of the arrays are the column names. The
+    print_table( $array_ref, [ \%options ] );
+
+The first argument must be a reference to an array of arrays. The first array of the arrays holds the column names. The
 following arrays are the table rows where the elements of these arrays are the field values.
 
-As a second and optional argument it can be passed a reference to a hash which holds the options as pairs of
+As a second and optional argument can be passed a reference to a hash which holds the options as pairs of
 "option-name" and "option-value".
 
 =head1 USAGE
